@@ -14,6 +14,8 @@
 #pragma _HP_SECONDARY_DEF PMPI_Attr_put  MPI_Attr_put
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Attr_put as PMPI_Attr_put
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val) __attribute__((weak,alias("PMPI_Attr_put")));
 #endif
 /* -- End Profiling Symbol Block */
 
