@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *
  *  (C) 2001 by Argonne National Laboratory.
@@ -14,6 +14,9 @@
 #pragma _HP_SECONDARY_DEF PMPI_Keyval_create  MPI_Keyval_create
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Keyval_create as PMPI_Keyval_create
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn,
+                      int *keyval, void *extra_state) __attribute__((weak,alias("PMPI_Keyval_create")));
 #endif
 /* -- End Profiling Symbol Block */
 
@@ -37,7 +40,7 @@ Input Parameters:
 . delete_fn - Delete callback function for 'keyval' 
 - extra_state - Extra state for callback functions 
 
-Output Parameter:
+Output Parameters:
 . keyval - key value for future access (integer) 
 
 Notes:
@@ -46,7 +49,7 @@ Key values are global (available for any and all communicators).
 There are subtle differences between C and Fortran that require that the
 copy_fn be written in the same language that 'MPI_Keyval_create'
 is called from.
-This should not be a problem for most users; only programers using both 
+This should not be a problem for most users; only programmers using both
 Fortran and C in the same program need to be sure that they follow this rule.
 
 .N ThreadSafe
