@@ -15,6 +15,8 @@
 #pragma _HP_SECONDARY_DEF PMPIX_Comm_remote_group_failed  MPIX_Comm_remote_group_failed
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPIX_Comm_remote_group_failed as PMPIX_Comm_remote_group_failed
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPIX_Comm_remote_group_failed(MPI_Comm comm, MPI_Group *failed_group) __attribute__((weak,alias("PMPIX_Comm_remote_group_failed")));
 #endif
 /* -- End Profiling Symbol Block */
 
