@@ -14,6 +14,9 @@
 #pragma _HP_SECONDARY_DEF PMPI_Recv  MPI_Recv
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Recv as PMPI_Recv
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag,
+             MPI_Comm comm, MPI_Status *status) __attribute__((weak,alias("PMPI_Recv")));
 #endif
 /* -- End Profiling Symbol Block */
 
