@@ -14,6 +14,11 @@
 #pragma _HP_SECONDARY_DEF PMPI_Sendrecv  MPI_Sendrecv
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Sendrecv as PMPI_Sendrecv
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest,
+                 int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype,
+                 int source, int recvtag, MPI_Comm comm, MPI_Status *status)
+                 __attribute__((weak,alias("PMPI_Sendrecv")));
 #endif
 /* -- End Profiling Symbol Block */
 
