@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *
  *  (C) 2001 by Argonne National Laboratory.
@@ -6,7 +6,6 @@
  */
 
 #include "mpiimpl.h"
-#include "rma.h"
 
 /* -- Begin Profiling Symbol Block for routine MPI_Win_wait */
 #if defined(HAVE_PRAGMA_WEAK)
@@ -15,6 +14,8 @@
 #pragma _HP_SECONDARY_DEF PMPI_Win_wait  MPI_Win_wait
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Win_wait as PMPI_Win_wait
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Win_wait(MPI_Win win) __attribute__((weak,alias("PMPI_Win_wait")));
 #endif
 /* -- End Profiling Symbol Block */
 
@@ -32,7 +33,7 @@
 /*@
    MPI_Win_wait - Completes an RMA exposure epoch begun with MPI_Win_post
 
-   Input Parameter:
+Input Parameters:
 . win - window object (handle) 
 
 .N ThreadSafe
