@@ -15,7 +15,9 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 
     *error_code = MPI_SUCCESS; // initialize to success
 
+#ifdef ROMIO_GPFS
     double startTimeBase,endTimeBase;
+#endif
 
     MPI_Status status;
     pthread_t io_thread;
@@ -29,7 +31,9 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
     int myAggRank = -1; // if I am an aggregor this is my index into fd->hints->ranklist
     int iAmUsedAgg = 0;
 
+#ifdef ROMIO_GPFS
     startTimeBase = MPI_Wtime();
+#endif
 
     int naggs = fd->hints->cb_nodes;
     int coll_bufsize = fd->hints->cb_buffer_size;
@@ -142,6 +146,7 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 
     MPI_Request *mpiSendDataToTargetAggRequest = (MPI_Request *) ADIOI_Malloc(numTargetAggs * sizeof(MPI_Request));
     MPI_Status mpiWaitAnyStatusFromTargetAggs,mpiWaitAnyStatusFromSourceProcs;
+    MPI_Status mpiIsendStatusForSize,  mpiIsendStatusForData;
 
     // use the write buffer allocated in the file_open
     char *write_buf0 = fd->io_buf;
@@ -172,12 +177,13 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 
     int *dataSizeGottenThisRoundPerProc = (int *)ADIOI_Malloc(numSourceProcs * sizeof(int));
     int *mpiRequestMapPerProc = (int *)ADIOI_Malloc(numSourceProcs * sizeof(int));
+    int *mpiSendRequestMapPerProc = (int *)ADIOI_Malloc(numTargetAggs * sizeof(int));
 
-    endTimeBase = MPI_Wtime();
 #ifdef ROMIO_GPFS
+    endTimeBase = MPI_Wtime();
     gpfsmpio_prof_cw[GPFSMPIO_CIO_T_MYREQ] += (endTimeBase-startTimeBase);
-#endif
     startTimeBase = MPI_Wtime();
+#endif
 
     /* each iteration of this loop writes a coll_bufsize portion of the file
      * domain */
@@ -197,10 +203,12 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 	}
 
 	int numRecvToWaitFor = 0;
-	int irecv;
+	int irecv,isend;
 
 	/* the source procs receive the amount of data the aggs want them to send */
+#ifdef ROMIO_GPFS
 	startTimeBase = MPI_Wtime();
+#endif
 	for (i=0;i<numTargetAggs;i++) {
 	    MPI_Irecv(&amountOfDataReqestedByTargetAgg[i],1,
 		    MPI_INT,targetAggsForMyData[i],0,
@@ -236,7 +244,9 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 
 	}
 
-	// the source procs send the requested data to the aggs - only send if requested more than 0 bytes
+        int numDataSendToWaitFor = 0;
+	/* the source procs send the requested data to the aggs - only send if
+	 * requested more than 0 bytes */
 	for (i = 0; i < numRecvToWaitFor; i++) {
 	    MPI_Waitany(numRecvToWaitFor,mpiSizeToSendRequest,&irecv,&mpiWaitAnyStatusFromTargetAggs);
 
@@ -249,14 +259,16 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 			targetAggsForMyData[irecv],0,fd->comm,&mpiSendDataToTargetAggRequest[irecv]);
 		totalAmountDataSent += amountOfDataReqestedByTargetAgg[irecv];
 		bufferOffsetToSendPerTargetAgg[irecv] += amountOfDataReqestedByTargetAgg[irecv];
+                mpiSendRequestMapPerProc[numDataSendToWaitFor] = irecv;
+                numDataSendToWaitFor++;
 	    }
 
 	}
 
 #ifdef ROMIO_GPFS
 	gpfsmpio_prof_cw[GPFSMPIO_CIO_T_DEXCH_SETUP] += (endTimeBase-startTimeBase);
-#endif
 	startTimeBase = MPI_Wtime();
+#endif
 
 	// the aggs receive the data from the source procs
 	int numDataRecvToWaitFor = 0;
@@ -302,8 +314,16 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 
 	}
 
-	endTimeBase = MPI_Wtime();
+	/* clean up the MPI_Request object for the MPI_Isend which told the
+	 * source procs how much data to send */
+        for (i=0;i<numSourceProcs;i++) {
+           MPI_Waitany(numSourceProcs,mpiSendDataSizeRequest,
+		   &isend,&mpiIsendStatusForSize);
+        }
+
+
 #ifdef ROMIO_GPFS
+        endTimeBase = MPI_Wtime();
 	gpfsmpio_prof_cw[GPFSMPIO_CIO_T_DEXCH_NET] += (endTimeBase-startTimeBase);
 #endif
 	// the aggs now write the data
@@ -355,11 +375,15 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
 
 	if (iAmUsedAgg)
 	    currentRoundFDStart += coll_bufsize;
+        for (i = 0; i < numDataSendToWaitFor; i++) {
+          MPI_Wait(&mpiSendDataToTargetAggRequest[mpiSendRequestMapPerProc[i]],
+		  &mpiIsendStatusForData);
+        }
 
     } // for-loop roundIter
 
-    endTimeBase=MPI_Wtime();
 #ifdef ROMIO_GPFS
+    endTimeBase = MPI_Wtime();
     gpfsmpio_prof_cw[GPFSMPIO_CIO_T_DEXCH] += (endTimeBase-startTimeBase);
 #endif
 
@@ -390,6 +414,7 @@ void ADIOI_P2PContigWriteAggregation(ADIO_File fd,
     ADIOI_Free(mpiSendDataToTargetAggRequest);
     ADIOI_Free(dataSizeGottenThisRoundPerProc);
     ADIOI_Free(mpiRequestMapPerProc);
+    ADIOI_Free(mpiSendRequestMapPerProc);
 
     /* TODO: still need a barrier here? */
     MPI_Barrier(fd->comm);
@@ -407,14 +432,19 @@ void ADIOI_P2PContigReadAggregation(ADIO_File fd,
 
     *error_code = MPI_SUCCESS; // initialize to success
 
+#ifdef ROMIO_GPFS
     double startTimeBase,endTimeBase;
+#endif
 
     MPI_Status status;
     pthread_t io_thread;
     void *thread_ret;
     ADIOI_IO_ThreadFuncData io_thread_args;
 
+#ifdef ROMIO_GPFS
     startTimeBase = MPI_Wtime();
+#endif
+
     int nprocs,myrank;
     MPI_Comm_size(fd->comm, &nprocs);
     MPI_Comm_rank(fd->comm, &myrank);
@@ -533,7 +563,7 @@ void ADIOI_P2PContigReadAggregation(ADIO_File fd,
     MPI_Request *mpiSendDataSizeRequest = (MPI_Request *) ADIOI_Malloc(numTargetProcs * sizeof(MPI_Request));
 
     MPI_Request *mpiSendDataToTargetProcRequest = (MPI_Request *) ADIOI_Malloc(numTargetProcs * sizeof(MPI_Request));
-    MPI_Status mpiWaitAnyStatusFromTargetAggs,mpiWaitAnyStatusFromSourceProcs;
+    MPI_Status mpiWaitAnyStatusFromTargetAggs,mpiWaitAnyStatusFromSourceProcs,mpiIsendStatusForSize,mpiIsendStatusForData;
 
     /* use the two-phase buffer allocated in the file_open - no app should ever
      * be both reading and writing at the same time */
@@ -569,8 +599,8 @@ void ADIOI_P2PContigReadAggregation(ADIO_File fd,
     }
 #endif
 
-    endTimeBase = MPI_Wtime();
 #ifdef ROMIO_GPFS
+    endTimeBase = MPI_Wtime();
     gpfsmpio_prof_cw[GPFSMPIO_CIO_T_MYREQ] += (endTimeBase-startTimeBase);
 #endif
 
@@ -579,7 +609,7 @@ void ADIOI_P2PContigReadAggregation(ADIO_File fd,
     int roundIter;
     for (roundIter=0;roundIter<numberOfRounds;roundIter++) {
 
-	int irecv;
+	int irecv,isend;
 	// determine what offsets define the portion of the file domain the agg is reading this round
 	if (iAmUsedAgg) {
 
@@ -601,7 +631,9 @@ void ADIOI_P2PContigReadAggregation(ADIO_File fd,
 			MPI_BYTE, ADIO_EXPLICIT_OFFSET, currentRoundFDStart,
 			&status, error_code);
 
+#ifdef ROMIO_GPFS
 		endTimeBase = MPI_Wtime();
+#endif
 	    }
 
 	    if (useIOBuffer) { // use the thread reader for the next round
@@ -755,6 +787,16 @@ void ADIOI_P2PContigReadAggregation(ADIO_File fd,
 
 	nextRoundFDStart = currentRoundFDStart + coll_bufsize;
 
+
+        // clean up the MPI_Isend MPI_Requests
+        for (i=0;i<numTargetProcs;i++) {
+          MPI_Waitany(numTargetProcs,mpiSendDataSizeRequest,
+		  &isend,&mpiIsendStatusForSize);
+          if (dataSizeSentThisRoundPerProc[isend] > 0) {
+            MPI_Wait(&mpiSendDataToTargetProcRequest[isend],&mpiIsendStatusForData);
+          }
+        }
+
 	MPI_Barrier(fd->comm); // need to sync up the source aggs which did the isend with the target procs which did the irecvs to give the target procs time to get the data before overwriting with next round readcontig
 
     } // for-loop roundIter
