@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *
  *  (C) 2001 by Argonne National Laboratory.
@@ -15,6 +15,8 @@
 #pragma _HP_SECONDARY_DEF PMPI_Cart_sub  MPI_Cart_sub
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Cart_sub as PMPI_Cart_sub
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *newcomm) __attribute__((weak,alias("PMPI_Cart_sub")));
 #endif
 /* -- End Profiling Symbol Block */
 
@@ -41,7 +43,7 @@ Input Parameters:
 dimension is kept in the subgrid (true) or is dropped (false) (logical 
 vector) 
 
-Output Parameter:
+Output Parameters:
 . newcomm - communicator containing the subgrid that includes the calling 
 process (handle) 
 
@@ -55,7 +57,7 @@ process (handle)
 .N MPI_ERR_COMM
 .N MPI_ERR_ARG
 @*/
-int MPI_Cart_sub(MPI_Comm comm, MPICH2_CONST int *remain_dims, MPI_Comm *comm_new)
+int MPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *newcomm)
 {
     int mpi_errno = MPI_SUCCESS, all_false;
     int ndims, key, color, ndims_in_subcomm, nnodes_in_subcomm, i, j, rank;
@@ -120,7 +122,7 @@ int MPI_Cart_sub(MPI_Comm comm, MPICH2_CONST int *remain_dims, MPI_Comm *comm_ne
     if (all_false) {
         /* ndims=0, or all entries in remain_dims are false.
            MPI 2.1 says return a 0D Cartesian topology. */
-	mpi_errno = MPIR_Cart_create_impl(comm_ptr, 0, NULL, NULL, 0, comm_new);
+	mpi_errno = MPIR_Cart_create_impl(comm_ptr, 0, NULL, NULL, 0, newcomm);
         if (mpi_errno) MPIU_ERR_POP(mpi_errno);
     } else {
 	/* Determine the number of remaining dimensions */
@@ -150,7 +152,7 @@ int MPI_Cart_sub(MPI_Comm comm, MPICH2_CONST int *remain_dims, MPI_Comm *comm_ne
 	mpi_errno = MPIR_Comm_split_impl( comm_ptr, color, key, &newcomm_ptr );
         if (mpi_errno) MPIU_ERR_POP(mpi_errno);
 
-        *comm_new = newcomm_ptr->handle;
+        *newcomm = newcomm_ptr->handle;
 	
 	/* Save the topology of this new communicator */
 	MPIU_CHKPMEM_MALLOC(toponew_ptr,MPIR_Topology*,sizeof(MPIR_Topology),
@@ -207,7 +209,7 @@ int MPI_Cart_sub(MPI_Comm comm, MPICH2_CONST int *remain_dims, MPI_Comm *comm_ne
     {
 	mpi_errno = MPIR_Err_create_code(
 	    mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER, "**mpi_cart_sub",
-	    "**mpi_cart_sub %C %p %p", comm, remain_dims, comm_new);
+	    "**mpi_cart_sub %C %p %p", comm, remain_dims, newcomm);
     }
 #   endif
     mpi_errno = MPIR_Err_return_comm( comm_ptr, FCNAME, mpi_errno );
