@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *
  *  (C) 2001 by Argonne National Laboratory.
@@ -15,6 +15,8 @@
 #pragma _HP_SECONDARY_DEF PMPI_Graph_neighbors_count  MPI_Graph_neighbors_count
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Graph_neighbors_count as PMPI_Graph_neighbors_count
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors) __attribute__((weak,alias("PMPI_Graph_neighbors_count")));
 #endif
 /* -- End Profiling Symbol Block */
 
@@ -66,7 +68,7 @@ Input Parameters:
 + comm - communicator with graph topology (handle) 
 - rank - rank of process in group of 'comm' (integer) 
 
-Output Parameter:
+Output Parameters:
 . nneighbors - number of neighbors of specified process (integer) 
 
 .N SignalSafe
