@@ -4,19 +4,12 @@
  *      See COPYRIGHT in top-level directory.
  */
 
-#include "mpidi_ch3_impl.h"
 #include "mpidrma.h"
 
 static int enableShortACC=1;
 
-#ifdef USE_MPIU_INSTR
-MPIU_INSTR_DURATION_EXTERN_DECL(wincreate_allgather);
-MPIU_INSTR_DURATION_EXTERN_DECL(winfree_rs);
-MPIU_INSTR_DURATION_EXTERN_DECL(winfree_complete);
-MPIU_INSTR_DURATION_EXTERN_DECL(rmaqueue_alloc);
-MPIU_INSTR_DURATION_EXTERN_DECL(rmaqueue_set);
-extern void MPIDI_CH3_RMA_InitInstr(void);
-#endif
+MPIR_T_PVAR_DOUBLE_TIMER_DECL_EXTERN(RMA, rma_rmaqueue_alloc);
+MPIR_T_PVAR_DOUBLE_TIMER_DECL_EXTERN(RMA, rma_rmaqueue_set);
 
 #define MPIDI_PASSIVE_TARGET_DONE_TAG  348297
 #define MPIDI_PASSIVE_TARGET_RMA_TAG 563924
@@ -44,10 +37,9 @@ extern void MPIDI_CH3_RMA_InitInstr(void);
 #define FCNAME MPIDI_QUOTE(FUNCNAME)
 int MPIDI_Win_free(MPID_Win **win_ptr)
 {
-    int mpi_errno=MPI_SUCCESS, total_pt_rma_puts_accs;
+    int mpi_errno=MPI_SUCCESS;
     int in_use;
     MPID_Comm *comm_ptr;
-    int errflag = FALSE;
     MPIDI_STATE_DECL(MPID_STATE_MPIDI_WIN_FREE);
         
     MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_WIN_FREE);
@@ -55,37 +47,10 @@ int MPIDI_Win_free(MPID_Win **win_ptr)
     MPIU_ERR_CHKANDJUMP((*win_ptr)->epoch_state != MPIDI_EPOCH_NONE,
                         mpi_errno, MPI_ERR_RMA_SYNC, "**rmasync");
 
+    mpi_errno = MPIDI_CH3I_Wait_for_pt_ops_finish(*win_ptr);
+    if(mpi_errno) MPIU_ERR_POP(mpi_errno);
+
     comm_ptr = (*win_ptr)->comm_ptr;
-    MPIU_INSTR_DURATION_START(winfree_rs);
-    mpi_errno = MPIR_Reduce_scatter_block_impl((*win_ptr)->pt_rma_puts_accs, 
-                                               &total_pt_rma_puts_accs, 1, 
-                                               MPI_INT, MPI_SUM, comm_ptr, &errflag);
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-    MPIU_ERR_CHKANDJUMP(errflag, mpi_errno, MPI_ERR_OTHER, "**coll_fail");
-    MPIU_INSTR_DURATION_END(winfree_rs);
-
-    if (total_pt_rma_puts_accs != (*win_ptr)->my_pt_rma_puts_accs)
-    {
-	MPID_Progress_state progress_state;
-            
-	/* poke the progress engine until the two are equal */
-	MPIU_INSTR_DURATION_START(winfree_complete);
-	MPID_Progress_start(&progress_state);
-	while (total_pt_rma_puts_accs != (*win_ptr)->my_pt_rma_puts_accs)
-	{
-	    mpi_errno = MPID_Progress_wait(&progress_state);
-	    /* --BEGIN ERROR HANDLING-- */
-	    if (mpi_errno != MPI_SUCCESS)
-	    {
-		MPID_Progress_end(&progress_state);
-		MPIU_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**winnoprogress");
-	    }
-	    /* --END ERROR HANDLING-- */
-	}
-	MPID_Progress_end(&progress_state);
-	MPIU_INSTR_DURATION_END(winfree_complete);
-    }
-
     mpi_errno = MPIR_Comm_free_impl(comm_ptr);
     if (mpi_errno) MPIU_ERR_POP(mpi_errno);
 
@@ -97,8 +62,10 @@ int MPIDI_Win_free(MPID_Win **win_ptr)
     MPIU_Free((*win_ptr)->pt_rma_puts_accs);
 
     /* Free the attached buffer for windows created with MPI_Win_allocate() */
-    if ((*win_ptr)->create_flavor == MPI_WIN_FLAVOR_ALLOCATE && (*win_ptr)->size > 0) {
-      MPIU_Free((*win_ptr)->base);
+    if ((*win_ptr)->create_flavor == MPI_WIN_FLAVOR_ALLOCATE || (*win_ptr)->create_flavor == MPI_WIN_FLAVOR_SHARED) {
+        if ((*win_ptr)->shm_allocated == FALSE && (*win_ptr)->size > 0) {
+            MPIU_Free((*win_ptr)->base);
+        }
     }
 
     MPIU_Object_release_ref(*win_ptr, &in_use);
@@ -115,37 +82,6 @@ int MPIDI_Win_free(MPID_Win **win_ptr)
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_SHM_Win_free
-#undef FCNAME
-#define FCNAME MPIDI_QUOTE(FUNCNAME)
-int MPIDI_SHM_Win_free(MPID_Win **win_ptr)
-{
-    int mpi_errno = MPI_SUCCESS;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_SHM_WIN_FREE);
-
-    MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_SHM_WIN_FREE);
-
-    /* Free memory allocated by the default shared memory window
-       implementation.  Note that this implementation works only for
-       MPI_COMM_SELF and does not map a shared segment. */
-
-    MPIU_Free((*win_ptr)->base);
-    MPIU_Free((*win_ptr)->shm_base_addrs);
-
-    mpi_errno = MPIDI_Win_free(win_ptr);
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-
- fn_exit:
-    MPIDI_RMA_FUNC_EXIT(MPID_STATE_MPIDI_SHM_WIN_FREE);
-    return mpi_errno;
-    /* --BEGIN ERROR HANDLING-- */
- fn_fail:
-    goto fn_exit;
-    /* --END ERROR HANDLING-- */
-}
-
-
 #undef FUNCNAME
 #define FUNCNAME MPIDI_Win_shared_query
 #undef FCNAME
@@ -158,7 +94,7 @@ int MPIDI_Win_shared_query(MPID_Win *win_ptr, int target_rank, MPI_Aint *size,
     MPIDI_STATE_DECL(MPID_STATE_MPIDI_WIN_SHARED_QUERY);
     MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_WIN_SHARED_QUERY);
 
-    *(void**) baseptr = win_ptr->shm_base_addrs[0];
+    *(void**) baseptr = win_ptr->base;
     *size             = win_ptr->size;
     *disp_unit        = win_ptr->disp_unit;
 
@@ -181,10 +117,11 @@ int MPIDI_Put(const void *origin_addr, int origin_count, MPI_Datatype
             int target_count, MPI_Datatype target_datatype, MPID_Win *win_ptr)
 {
     int mpi_errno = MPI_SUCCESS;
-    int dt_contig ATTRIBUTE((unused)), rank, predefined;
+    int dt_contig ATTRIBUTE((unused)), rank;
     MPID_Datatype *dtp;
     MPI_Aint dt_true_lb ATTRIBUTE((unused));
     MPIDI_msg_sz_t data_sz;
+    MPIDI_VC_t *orig_vc = NULL, *target_vc = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIDI_PUT);
         
     MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_PUT);
@@ -207,27 +144,29 @@ int MPIDI_Put(const void *origin_addr, int origin_count, MPI_Datatype
 	goto fn_exit;
     }
 
-    rank = win_ptr->myrank;
+    rank = win_ptr->comm_ptr->rank;
     
+    if (win_ptr->shm_allocated == TRUE && target_rank != rank && win_ptr->create_flavor != MPI_WIN_FLAVOR_SHARED) {
+        /* check if target is local and shared memory is allocated on window,
+           if so, we directly perform this operation on shared memory region. */
+
+        /* FIXME: Here we decide whether to perform SHM operations by checking if origin and target are on
+           the same node. However, in ch3:sock, even if origin and target are on the same node, they do
+           not within the same SHM region. Here we filter out ch3:sock by checking shm_allocated flag first,
+           which is only set to TRUE when SHM region is allocated in nemesis.
+           In future we need to figure out a way to check if origin and target are in the same "SHM comm".
+        */
+        MPIDI_Comm_get_vc(win_ptr->comm_ptr, rank, &orig_vc);
+        MPIDI_Comm_get_vc(win_ptr->comm_ptr, target_rank, &target_vc);
+    }
+
     /* If the put is a local operation, do it here */
-    if (target_rank == rank || win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED)
+    if (target_rank == rank || win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED ||
+        (win_ptr->shm_allocated == TRUE && orig_vc->node_id == target_vc->node_id))
     {
-        void *base;
-        int disp_unit;
-
-        if (win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED) {
-            base = win_ptr->shm_base_addrs[target_rank];
-            disp_unit = win_ptr->disp_units[target_rank];
-        }
-        else {
-            base = win_ptr->base;
-            disp_unit = win_ptr->disp_unit;
-        }
-
-        mpi_errno = MPIR_Localcopy(origin_addr, origin_count, origin_datatype,
-                                   (char *) base + disp_unit * target_disp,
-                                   target_count, target_datatype);
-        if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
+        mpi_errno = MPIDI_CH3I_Shm_put_op(origin_addr, origin_count, origin_datatype, target_rank,
+                                          target_disp, target_count, target_datatype, win_ptr);
+        if (mpi_errno) MPIU_ERR_POP(mpi_errno);
     }
     else
     {
@@ -235,12 +174,12 @@ int MPIDI_Put(const void *origin_addr, int origin_count, MPI_Datatype
         MPIDI_RMA_Op_t *new_ptr = NULL;
 
 	/* queue it up */
-        MPIU_INSTR_DURATION_START(rmaqueue_alloc);
+        MPIR_T_PVAR_TIMER_START(RMA, rma_rmaqueue_alloc);
         mpi_errno = MPIDI_CH3I_RMA_Ops_alloc_tail(ops_list, &new_ptr);
-        MPIU_INSTR_DURATION_END(rmaqueue_alloc);
+        MPIR_T_PVAR_TIMER_END(RMA, rma_rmaqueue_alloc);
         if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
 
-	MPIU_INSTR_DURATION_START(rmaqueue_set);
+	MPIR_T_PVAR_TIMER_START(RMA, rma_rmaqueue_set);
 	/* FIXME: For contig and very short operations, use a streamlined op */
 	new_ptr->type = MPIDI_RMA_PUT;
         /* Cast away const'ness for the origin address, as the
@@ -253,18 +192,16 @@ int MPIDI_Put(const void *origin_addr, int origin_count, MPI_Datatype
 	new_ptr->target_disp = target_disp;
 	new_ptr->target_count = target_count;
 	new_ptr->target_datatype = target_datatype;
-	MPIU_INSTR_DURATION_END(rmaqueue_set);
+	MPIR_T_PVAR_TIMER_END(RMA, rma_rmaqueue_set);
 
 	/* if source or target datatypes are derived, increment their
-	   reference counts */ 
-	MPIDI_CH3I_DATATYPE_IS_PREDEFINED(origin_datatype, predefined);
-	if (!predefined)
+	   reference counts */
+	if (!MPIR_DATATYPE_IS_PREDEFINED(origin_datatype))
 	{
 	    MPID_Datatype_get_ptr(origin_datatype, dtp);
 	    MPID_Datatype_add_ref(dtp);
 	}
-	MPIDI_CH3I_DATATYPE_IS_PREDEFINED(target_datatype, predefined);
-	if (!predefined)
+	if (!MPIR_DATATYPE_IS_PREDEFINED(target_datatype))
 	{
 	    MPID_Datatype_get_ptr(target_datatype, dtp);
 	    MPID_Datatype_add_ref(dtp);
@@ -293,9 +230,10 @@ int MPIDI_Get(void *origin_addr, int origin_count, MPI_Datatype
 {
     int mpi_errno = MPI_SUCCESS;
     MPIDI_msg_sz_t data_sz;
-    int dt_contig ATTRIBUTE((unused)), rank, predefined;
+    int dt_contig ATTRIBUTE((unused)), rank;
     MPI_Aint dt_true_lb ATTRIBUTE((unused));
     MPID_Datatype *dtp;
+    MPIDI_VC_t *orig_vc = NULL, *target_vc = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIDI_GET);
         
     MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_GET);
@@ -318,27 +256,29 @@ int MPIDI_Get(void *origin_addr, int origin_count, MPI_Datatype
 	goto fn_exit;
     }
 
-    rank = win_ptr->myrank;
+    rank = win_ptr->comm_ptr->rank;
+
+    if (win_ptr->shm_allocated == TRUE && target_rank != rank && win_ptr->create_flavor != MPI_WIN_FLAVOR_SHARED) {
+        /* check if target is local and shared memory is allocated on window,
+           if so, we directly perform this operation on shared memory region. */
+
+        /* FIXME: Here we decide whether to perform SHM operations by checking if origin and target are on
+           the same node. However, in ch3:sock, even if origin and target are on the same node, they do
+           not within the same SHM region. Here we filter out ch3:sock by checking shm_allocated flag first,
+           which is only set to TRUE when SHM region is allocated in nemesis.
+           In future we need to figure out a way to check if origin and target are in the same "SHM comm".
+        */
+        MPIDI_Comm_get_vc(win_ptr->comm_ptr, rank, &orig_vc);
+        MPIDI_Comm_get_vc(win_ptr->comm_ptr, target_rank, &target_vc);
+    }
     
     /* If the get is a local operation, do it here */
-    if (target_rank == rank || win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED)
+    if (target_rank == rank || win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED ||
+        (win_ptr->shm_allocated == TRUE && orig_vc->node_id == target_vc->node_id))
     {
-        void *base;
-        int disp_unit;
-
-        if (win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED) {
-            base = win_ptr->shm_base_addrs[target_rank];
-            disp_unit = win_ptr->disp_units[target_rank];
-        }
-        else {
-            base = win_ptr->base;
-            disp_unit = win_ptr->disp_unit;
-        }
-
-        mpi_errno = MPIR_Localcopy((char *) base + disp_unit * target_disp,
-                                   target_count, target_datatype, origin_addr,
-                                   origin_count, origin_datatype);
-        if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
+        mpi_errno = MPIDI_CH3I_Shm_get_op(origin_addr, origin_count, origin_datatype, target_rank,
+                                          target_disp, target_count, target_datatype, win_ptr);
+        if (mpi_errno) MPIU_ERR_POP(mpi_errno);
     }
     else
     {
@@ -346,12 +286,12 @@ int MPIDI_Get(void *origin_addr, int origin_count, MPI_Datatype
         MPIDI_RMA_Op_t *new_ptr = NULL;
 
 	/* queue it up */
-        MPIU_INSTR_DURATION_START(rmaqueue_alloc);
+        MPIR_T_PVAR_TIMER_START(RMA, rma_rmaqueue_alloc);
         mpi_errno = MPIDI_CH3I_RMA_Ops_alloc_tail(ops_list, &new_ptr);
-        MPIU_INSTR_DURATION_END(rmaqueue_alloc);
+        MPIR_T_PVAR_TIMER_END(RMA, rma_rmaqueue_alloc);
         if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
 
-	MPIU_INSTR_DURATION_START(rmaqueue_set);
+	MPIR_T_PVAR_TIMER_START(RMA, rma_rmaqueue_set);
 	/* FIXME: For contig and very short operations, use a streamlined op */
 	new_ptr->type = MPIDI_RMA_GET;
 	new_ptr->origin_addr = origin_addr;
@@ -361,18 +301,16 @@ int MPIDI_Get(void *origin_addr, int origin_count, MPI_Datatype
 	new_ptr->target_disp = target_disp;
 	new_ptr->target_count = target_count;
 	new_ptr->target_datatype = target_datatype;
-	MPIU_INSTR_DURATION_END(rmaqueue_set);
+	MPIR_T_PVAR_TIMER_END(RMA, rma_rmaqueue_set);
 	
 	/* if source or target datatypes are derived, increment their
-	   reference counts */ 
-	MPIDI_CH3I_DATATYPE_IS_PREDEFINED(origin_datatype, predefined);
-	if (!predefined)
+	   reference counts */
+	if (!MPIR_DATATYPE_IS_PREDEFINED(origin_datatype))
 	{
 	    MPID_Datatype_get_ptr(origin_datatype, dtp);
 	    MPID_Datatype_add_ref(dtp);
 	}
-	MPIDI_CH3I_DATATYPE_IS_PREDEFINED(target_datatype, predefined);
-	if (!predefined)
+	if (!MPIR_DATATYPE_IS_PREDEFINED(target_datatype))
 	{
 	    MPID_Datatype_get_ptr(target_datatype, dtp);
 	    MPID_Datatype_add_ref(dtp);
@@ -402,10 +340,10 @@ int MPIDI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype
 {
     int mpi_errno=MPI_SUCCESS;
     MPIDI_msg_sz_t data_sz;
-    int dt_contig ATTRIBUTE((unused)), rank, origin_predefined, target_predefined;
+    int dt_contig ATTRIBUTE((unused)), rank;
     MPI_Aint dt_true_lb ATTRIBUTE((unused));
     MPID_Datatype *dtp;
-    MPIU_CHKLMEM_DECL(2);
+    MPIDI_VC_t *orig_vc = NULL, *target_vc = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIDI_ACCUMULATE);
     
     MPIDI_RMA_FUNC_ENTER(MPID_STATE_MPIDI_ACCUMULATE);
@@ -428,133 +366,30 @@ int MPIDI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype
 	goto fn_exit;
     }
 
-    rank = win_ptr->myrank;
+    rank = win_ptr->comm_ptr->rank;
     
-    MPIDI_CH3I_DATATYPE_IS_PREDEFINED(origin_datatype, origin_predefined);
-    MPIDI_CH3I_DATATYPE_IS_PREDEFINED(target_datatype, target_predefined);
+    if (win_ptr->shm_allocated == TRUE && target_rank != rank && win_ptr->create_flavor != MPI_WIN_FLAVOR_SHARED) {
+        /* check if target is local and shared memory is allocated on window,
+           if so, we directly perform this operation on shared memory region. */
+
+        /* FIXME: Here we decide whether to perform SHM operations by checking if origin and target are on
+           the same node. However, in ch3:sock, even if origin and target are on the same node, they do
+           not within the same SHM region. Here we filter out ch3:sock by checking shm_allocated flag first,
+           which is only set to TRUE when SHM region is allocated in nemesis.
+           In future we need to figure out a way to check if origin and target are in the same "SHM comm".
+        */
+        MPIDI_Comm_get_vc(win_ptr->comm_ptr, rank, &orig_vc);
+        MPIDI_Comm_get_vc(win_ptr->comm_ptr, target_rank, &target_vc);
+    }
 
     /* Do =! rank first (most likely branch?) */
-    if (target_rank == rank || win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED)
+    if (target_rank == rank || win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED ||
+        (win_ptr->shm_allocated == TRUE && orig_vc->node_id == target_vc->node_id))
     {
-        MPI_User_function *uop;
-        void *base;
-        int disp_unit, shm_op = 0;
-
-        if (win_ptr->create_flavor == MPI_WIN_FLAVOR_SHARED) {
-            shm_op = 1;
-            base = win_ptr->shm_base_addrs[target_rank];
-            disp_unit = win_ptr->disp_units[target_rank];
-        }
-        else {
-            base = win_ptr->base;
-            disp_unit = win_ptr->disp_unit;
-        }
-	
-	if (op == MPI_REPLACE)
-	{
-            if (shm_op) MPIDI_CH3I_SHM_MUTEX_LOCK(win_ptr);
-            mpi_errno = MPIR_Localcopy(origin_addr, origin_count,
-                                       origin_datatype,
-                                       (char *) base + disp_unit * target_disp,
-                                       target_count, target_datatype);
-            if (shm_op) MPIDI_CH3I_SHM_MUTEX_UNLOCK(win_ptr);
-            if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-	    goto fn_exit;
-	}
-	
-	MPIU_ERR_CHKANDJUMP1((HANDLE_GET_KIND(op) != HANDLE_KIND_BUILTIN), 
-			     mpi_errno, MPI_ERR_OP, "**opnotpredefined",
-			     "**opnotpredefined %d", op );
-	
-	/* get the function by indexing into the op table */
-	uop = MPIR_OP_HDL_TO_FN(op);
-	
-	if (origin_predefined && target_predefined)
-	{    
-            /* Cast away const'ness for origin_address in order to
-             * avoid changing the prototype for MPI_User_function */
-            if (shm_op) MPIDI_CH3I_SHM_MUTEX_LOCK(win_ptr);
-            (*uop)((void *) origin_addr, (char *) base + disp_unit*target_disp,
-                   &target_count, &target_datatype);
-            if (shm_op) MPIDI_CH3I_SHM_MUTEX_UNLOCK(win_ptr);
-	}
-	else
-	{
-	    /* derived datatype */
-	    
-	    MPID_Segment *segp;
-	    DLOOP_VECTOR *dloop_vec;
-	    MPI_Aint first, last;
-	    int vec_len, i, type_size, count;
-	    MPI_Datatype type;
-	    MPI_Aint true_lb, true_extent, extent;
-	    void *tmp_buf=NULL, *target_buf;
-            const void *source_buf;
-	    
-	    if (origin_datatype != target_datatype)
-	    {
-		/* first copy the data into a temporary buffer with
-		   the same datatype as the target. Then do the
-		   accumulate operation. */
-		
-		MPIR_Type_get_true_extent_impl(target_datatype, &true_lb, &true_extent);
-		MPID_Datatype_get_extent_macro(target_datatype, extent); 
-		
-		MPIU_CHKLMEM_MALLOC(tmp_buf, void *, 
-			target_count * (MPIR_MAX(extent,true_extent)), 
-			mpi_errno, "temporary buffer");
-		/* adjust for potential negative lower bound in datatype */
-		tmp_buf = (void *)((char*)tmp_buf - true_lb);
-		
-		mpi_errno = MPIR_Localcopy(origin_addr, origin_count,
-					   origin_datatype, tmp_buf,
-					   target_count, target_datatype);  
-		if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-	    }
-
-	    if (target_predefined) { 
-		/* target predefined type, origin derived datatype */
-
-                if (shm_op) MPIDI_CH3I_SHM_MUTEX_LOCK(win_ptr);
-                (*uop)(tmp_buf, (char *) base + disp_unit * target_disp,
-                       &target_count, &target_datatype);
-                if (shm_op) MPIDI_CH3I_SHM_MUTEX_UNLOCK(win_ptr);
-	    }
-	    else {
-	    
-		segp = MPID_Segment_alloc();
-		MPIU_ERR_CHKANDJUMP1((!segp), mpi_errno, MPI_ERR_OTHER, 
-				    "**nomem","**nomem %s","MPID_Segment_alloc"); 
-		MPID_Segment_init(NULL, target_count, target_datatype, segp, 0);
-		first = 0;
-		last  = SEGMENT_IGNORE_LAST;
-		
-		MPID_Datatype_get_ptr(target_datatype, dtp);
-		vec_len = dtp->max_contig_blocks * target_count + 1; 
-		/* +1 needed because Rob says so */
-		MPIU_CHKLMEM_MALLOC(dloop_vec, DLOOP_VECTOR *, 
-				    vec_len * sizeof(DLOOP_VECTOR), 
-				    mpi_errno, "dloop vector");
-		
-		MPID_Segment_pack_vector(segp, first, &last, dloop_vec, &vec_len);
-		
-		source_buf = (tmp_buf != NULL) ? tmp_buf : origin_addr;
-		target_buf = (char *) base + disp_unit * target_disp;
-		type = dtp->eltype;
-		type_size = MPID_Datatype_get_basic_size(type);
-                if (shm_op) MPIDI_CH3I_SHM_MUTEX_LOCK(win_ptr);
-		for (i=0; i<vec_len; i++)
-		{
-		    count = (dloop_vec[i].DLOOP_VECTOR_LEN)/type_size;
-		    (*uop)((char *)source_buf + MPIU_PtrToAint(dloop_vec[i].DLOOP_VECTOR_BUF),
-			   (char *)target_buf + MPIU_PtrToAint(dloop_vec[i].DLOOP_VECTOR_BUF),
-			   &count, &type);
-		}
-                if (shm_op) MPIDI_CH3I_SHM_MUTEX_UNLOCK(win_ptr);
-		
-		MPID_Segment_free(segp);
-	    }
-	}
+	mpi_errno = MPIDI_CH3I_Shm_acc_op(origin_addr, origin_count, origin_datatype,
+					  target_rank, target_disp, target_count, target_datatype,
+					  op, win_ptr);
+	if (mpi_errno) MPIU_ERR_POP(mpi_errno);
     }
     else
     {
@@ -562,14 +397,15 @@ int MPIDI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype
         MPIDI_RMA_Op_t *new_ptr = NULL;
 
 	/* queue it up */
-        MPIU_INSTR_DURATION_START(rmaqueue_alloc);
+        MPIR_T_PVAR_TIMER_START(RMA, rma_rmaqueue_alloc);
         mpi_errno = MPIDI_CH3I_RMA_Ops_alloc_tail(ops_list, &new_ptr);
-        MPIU_INSTR_DURATION_END(rmaqueue_alloc);
+        MPIR_T_PVAR_TIMER_END(RMA, rma_rmaqueue_alloc);
         if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
 
 	/* If predefined and contiguous, use a simplified element */
-	if (origin_predefined && target_predefined && enableShortACC) {
-	    MPIU_INSTR_DURATION_START(rmaqueue_set);
+	if (MPIR_DATATYPE_IS_PREDEFINED(origin_datatype) &&
+            MPIR_DATATYPE_IS_PREDEFINED(target_datatype) && enableShortACC) {
+	    MPIR_T_PVAR_TIMER_START(RMA, rma_rmaqueue_set);
 	    new_ptr->type = MPIDI_RMA_ACC_CONTIG;
 	    /* Only the information needed for the contig/predefined acc */
             /* Cast away const'ness for origin_address as
@@ -582,11 +418,11 @@ int MPIDI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype
 	    new_ptr->target_count = target_count;
 	    new_ptr->target_datatype = target_datatype;
 	    new_ptr->op = op;
-	    MPIU_INSTR_DURATION_END(rmaqueue_set);
+	    MPIR_T_PVAR_TIMER_END(RMA, rma_rmaqueue_set);
 	    goto fn_exit;
 	}
 
-	MPIU_INSTR_DURATION_START(rmaqueue_set);
+	MPIR_T_PVAR_TIMER_START(RMA, rma_rmaqueue_set);
 	new_ptr->type = MPIDI_RMA_ACCUMULATE;
         /* Cast away const'ness for origin_address as MPIDI_RMA_Op_t
          * contain both PUT and GET like ops */
@@ -598,16 +434,16 @@ int MPIDI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype
 	new_ptr->target_count = target_count;
 	new_ptr->target_datatype = target_datatype;
 	new_ptr->op = op;
-	MPIU_INSTR_DURATION_END(rmaqueue_set);
+	MPIR_T_PVAR_TIMER_END(RMA, rma_rmaqueue_set);
 	
 	/* if source or target datatypes are derived, increment their
-	   reference counts */ 
-	if (!origin_predefined)
+	   reference counts */
+	if (!MPIR_DATATYPE_IS_PREDEFINED(origin_datatype))
 	{
 	    MPID_Datatype_get_ptr(origin_datatype, dtp);
 	    MPID_Datatype_add_ref(dtp);
 	}
-	if (!target_predefined)
+	if (!MPIR_DATATYPE_IS_PREDEFINED(target_datatype))
 	{
 	    MPID_Datatype_get_ptr(target_datatype, dtp);
 	    MPID_Datatype_add_ref(dtp);
@@ -615,7 +451,6 @@ int MPIDI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype
     }
 
  fn_exit:
-    MPIU_CHKLMEM_FREEALL();
     MPIDI_RMA_FUNC_EXIT(MPID_STATE_MPIDI_ACCUMULATE);
     return mpi_errno;
 
