@@ -25,12 +25,12 @@ Output Parameters:
 
 int MPIR_Type_flatten(MPI_Datatype type,
 		      MPI_Aint *off_array,
-		      int *size_array,
+		      MPI_Aint *size_array,
 		      MPI_Aint *array_len_p)
 {
     int err;
     MPI_Aint first, last;
-    MPID_Datatype *datatype_ptr;
+    MPID_Datatype *datatype_ptr ATTRIBUTE((unused));
     MPID_Segment *segp;
 
     if (HANDLE_GET_KIND(type) == HANDLE_KIND_BUILTIN) {
